/*
 * Classname: AuthenticationService
 * Version: 1.11
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package authentication;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPMessage;

/**
 * This class is the main class for the authentication service. It uses the other classes to
 * consider an authentication request and returns the result.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */

public class AuthenticationService {

    public static final boolean DEBUG = true;

    /**
     * This method can be used to authenticate a user by calling it with a SASL request inside
     * an HTTP request. It will return a String representation of a SASL response.
     * 
     * @param req HttpServletRequest The HTTP request containing the authentication request.
     * @return String The response to the request.
     */
    public static String authenticate(HttpServletRequest req) {
        ServletInputStream sis = null;
        String s = "";
        try {
            sis = req.getInputStream();
            MessageFactory msgFact = MessageFactory.newInstance();
            SOAPMessage doc = msgFact.createMessage(new MimeHeaders(), sis);
            SaslRequest sreq = null;
            String status = null;
            ResourceOffering offering = null;
            try {
                sreq = new SaslRequest(doc);
                UserDb data = new UserDb();
                boolean authenticated = data
                        .authenticate(sreq.getUserID(), sreq.getPassword());
                if (authenticated) {
                    /* The authentication was successful. */
                    offering = data.readOffering();
                    status = SaslResponse.STATUS_OK;
                } else {
                    /* The authentication failed. */
                    status = SaslResponse.STATUS_ABORT;
                }
            } catch (Exception e) {
                /* Something wrong happened. */
                if (DEBUG) Log.write(e);
                status = SaslResponse.STATUS_ABORT;
            }
            /* Create the response: */
            SaslResponse sres = new SaslResponse(sreq.getMessageID(), status, 
                    sreq.getMechanism(), offering);
            s = sres.toString();
            if (DEBUG) debug(sreq, sres);
        } catch (Exception e) {
            /* Something wrong happened. */
            if (DEBUG) Log.write(e);
        } finally {
            /* Clean up: */
            try {
                if (sis != null) sis.close();
            } catch (IOException e) {
                if (DEBUG) Log.write(e);
            }
        }
        return s;
    }

    /**
     * This method can be used to write the request and response message to a text file so that
     * they can be examined to se if they are correct.
     * 
     * @param req SaslRequest The SASL request receved in the HTTP request. May be null if
     *            something went wrong during the parsing.
     * @param res SaslResponse The SASL response as it will be returned to the client.
     */
    private static void debug(SaslRequest req, SaslResponse res) {
        try {
            File f = new File("/home/apache-tomcat-5.5.17/logs/ASdebug.txt");
            if (!f.exists()) f.createNewFile();
            FileOutputStream fos = new FileOutputStream(f);
            String s = "MessageID: " + req.getMessageID() + "\n";
            fos.write("-----------Request: ----------\n".getBytes());
            fos.write(s.getBytes());
            s = "Mechanism: " + req.getMechanism() + "\n";
            fos.write(s.getBytes());
            s = "UserID: " + req.getUserID() + "\n";
            fos.write(s.getBytes());
            s = "Timestamp: " + req.getTimestamp() + "\n";
            fos.write(s.getBytes());
            s = req.toString();
            fos.write(s.getBytes());
            fos.write("\n-----------Response: ----------\n".getBytes());
            s = res.toString();
            fos.write(s.getBytes());
            fos.flush();
        } catch (Exception e) {
            Log.write(e);
        }
    }

}
