/*
 * Classname: Log
 * Version: 1.3
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package authentication;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

/**
 * The Log class is used during debugging. It creates a log of exceptions that has occured.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class Log {

    /**
     * This method writes the message, class and stack trace of the Exception to the log file.
     * 
     * @param exception Exception The exception that should be written to the log.
     */
    public static void write(Exception exception) {
        BufferedWriter writer = null;
        try {
            File logFile = new File("/home/apache-tomcat-5.5.17/logs/ASlog.txt");
            if (!logFile.exists()) {
                logFile.createNewFile();
            }
            writer = new BufferedWriter(new FileWriter(logFile, true));
            writer.write("- " + new Date(System.currentTimeMillis()).toString()
                    + " ----------------------------------------------------------");
            writer.newLine();
            writer.write(exception.getClass() + ": " + exception.getMessage());
            writer.newLine();
            String classname;
            String methodname;
            int lineno;
            StackTraceElement[] trace = exception.getStackTrace();
            if (trace != null) {
                for (int i = 0; i < trace.length; i++) {
                    classname = trace[i].getClassName();
                    methodname = trace[i].getMethodName();
                    lineno = trace[i].getLineNumber();
                    writer.write("     Class: " + classname + " Method: " + methodname
                            + " Lineno: " + lineno);
                    writer.newLine();
                }
            }
            writer.write("------------------------------------------------------------"
                    + "------------------------------");
            writer.newLine();
            writer.flush();
        } catch (Exception e) {
        } finally {
            /* Clean up: */
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                }
            }
        }
    }
}
