/*
 * Classname: Randomizer
 * Version: 1.4
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package authentication;

import java.security.SecureRandom;

/**
 * This class is used to create unique identifiers. The probability of two identifiers being
 * identical should be 1/P_GOAL.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class Randomizer {

    private static final long P_GOAL = 2 ^ 128;
    private static SecureRandom rand = new SecureRandom();
    private static char[] values = new char[] { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
            'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y',
            'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
            'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4',
            '5', '6', '7', '8', '9' };

    /**
     * This method uses the default Random object to generate a unique number of appropriate length 
     * to satisfy the probability goal.
     * 
     * @return String A new identifier value.
     */
    public static synchronized String nextID() {
        StringBuffer id = new StringBuffer();
        while ((values.length ^ id.length()) < P_GOAL) {
            id.append(values[rand.nextInt(values.length)]);
        }
        return id.toString();
    }
}
