/*
 * Classname: UserDb
 * Version: 1.10
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package authentication;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This class is the interface to the XML file containing the list of users. It is used to
 * retrieve a user, check that the password is correct and get the resource offering for the
 * discovery service for that user.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class UserDb {

    private final String userdb = "as_db.xml";
    private ResourceOffering offering;

    /**
     * This method authenticates the user by checking that the username is identical with one
     * contained in the list of users and that the password is the same as the password of that
     * user. Then it retrieves the resource offering of that user and stores it in the offering
     * variable so that it can be returned later.
     * 
     * @param un String The user identifier of the user to be authenticated.
     * @param pwd String The password of the user to be authenticated.
     * @return boolean Returns true if the authentication was successful, false otherwise.
     */
    public boolean authenticate(String un, String pwd) {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(userdb);

            Element root = doc.getDocumentElement();
            NodeList users = root.getElementsByTagName("profile");
            Node user;

            for (int i = 0; i < users.getLength(); i++) {
                user = users.item(i);

                if (usernameFound(un, user)) {
                    if (correctPassword(pwd, user)) {
                        offering = new ResourceOffering(getOffering(user));
                        return true;
                    }
                }
            }
        } catch (Exception e) {
            if (AuthenticationService.DEBUG) Log.write(e);
        }

        return false;
    }

    /**
     * @return boolean Checks whether the password is correct for the current user. Returns
     *         true if it is and false otherwise.
     */
    private boolean correctPassword(String pwd, Node user) {
        return user.getChildNodes().item(3).getChildNodes().item(0).getNodeValue().equals(pwd);
    }

    /**
     * @return boolean Checks whether the user identifier is correct for the current user.
     *         Returns true if it is and false otherwise.
     */
    private boolean usernameFound(String un, Node user) {
        return user.getChildNodes().item(1).getChildNodes().item(0).getNodeValue().equals(un);
    }

    /**
     * @return Node Retrieves the user's resource offering for the discovery service.
     */
    private Node getOffering(Node user) {
        return user.getChildNodes().item(5);
    }

    /**
     * @return ResourceOffering The ResourceOffering object containing the user's resource
     *         offering for the discovery service.
     */
    public ResourceOffering readOffering() {
        return offering;
    }
}
