/*
 * Classname: ServiceOfferings
 * Version: 1.10
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package discovery;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * The ServiceOfferings class is used to access the service offerings which are stored in an
 * XML file. It only has one method which finds the ResourceOfferings for the current user.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class ServiceOfferings {

    private static final String dbFile = "ds_db.xml";

    /**
     * This method searches through the XML file to find the current user. If this user exists
     * and has any service offerings the method creates a ResourceOffering object for each
     * offering and returns these.
     * 
     * @return ResourceOffering[] List of this user's resource offerings
     */
    public static ResourceOffering[] readOfferings(String un) {
        try {
            Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(dbFile);
            Element root = doc.getDocumentElement();

            NodeList users = root.getElementsByTagName("profile");
            Node user;

            for (int i = 0; i < users.getLength(); i++) {
                user = users.item(i);
                if (usernameFound(user, un)) {
                    NodeList offersFound = ((Element) user).getElementsByTagName("ResourceOffering");
                    if (offersFound.getLength() > 0) {
                        ResourceOffering[] offerings = new ResourceOffering[offersFound.getLength()];
                        for (int j = 0; j < offersFound.getLength(); j++) {
                            offerings[j] = new ResourceOffering(offersFound.item(j));
                        }
                        return offerings;
                    } else {
                        return null;
                    }
                }
            }
        } catch (ParserConfigurationException pce) {
            if (DiscoveryService.DEBUG) Log.write(pce);
        } catch (IOException ioe) {
            if (DiscoveryService.DEBUG) Log.write(ioe);
        } catch (SAXException saxe) {
            if (DiscoveryService.DEBUG) Log.write(saxe);
        }

        return null;
    }

    /**
     * @return boolean Checks if the input Node contains the correct username as specified in
     *         the other input parameter. Returns true if it does and false otherwise.
     */
    private static boolean usernameFound(Node user, String un) {
        return user.getChildNodes().item(1).getChildNodes().item(0).getNodeValue().equals(un);
    }
}
