/*
 * Classname: MessageException
 * Version: 1.6
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package mymobilesite.communication;

import mymobilesite.controller.Language;

/**
 * This class is responsible for holding the exceptions encountered in the message classes.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class MessageException extends Exception {

    private int errorCode;

    /**
     * Constructor that creates an exception with a message based on the errorCode.
     * 
     * @param errorCode int The error code describing the situation. Must be one of the
     *        constants in Language beginning with "ERROR_".
     */
    public MessageException(int errorCode) {
        super(Language.getErrorMsg(errorCode));
        this.errorCode = errorCode;
    }

    /**
     * Constructor that creates an exception with a customised message.
     * 
     * @param errorCode int The error code describing the situation. Must be one of the
     *        constants in Language beginning with "ERROR_".
     */
    public MessageException(int errorCode, String msg) {
        super((msg != null ? msg : Language.getErrorMsg(errorCode)));
        this.errorCode = errorCode;
    }

    /**
     * @return int The error code for this exception.
     */
    public int getErrorCode() {
        return errorCode;
    }
}
