/*
 * Classname: Configurations
 * Version: 1.16
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package mymobilesite.controller;

import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

/**
 * The Configurations class must manage the things the user should be able to configure, which
 * is the username. This should be stored until the next time the application is used and the
 * language which can be changed. This class also holds static variables that are used by a
 * variety of classes and that are collected so that it is simpler to change their values
 * later.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class Configurations implements Runnable {

    public static final String IDP_URL = "https://fedoralab1.kantega.no:8443/AS/authentication.jsp";
    public static final String RECORDSTORE_NAME = "settings";

    public static final String SERVICE_TYPE_URI = 
        "http://fedoralab1.kantega.no:8080/abstract-mymobilesite-wsp/mymobilesite-wsp.wsdl";
    public static final String REQUEST_SCHEMA_URI = 
        "http://fedoralab1.kantega.no:8080/requests/mymobilesite-request.xsd";
    public static final String RESPONSE_SCHEMA_URI = 
        "http://fedoralab1.kantega.no:8080/responses/mymobilesite-response.xsd";
    public static final String FAULT_SCHEMA_URI = 
        "http://fedoralab1.kantega.no:8080/faults/mymobilesite-faults.xsd";

    public static final int NAME_LENGTH_CONSTRAINT = 25;
    public static final long TIMESTAMP_DEVIATION_MILLIS = 600000;

    /** These variables gives the application's default values for language and username. */
    public static String LANGUAGE = "gb";
    public static String USERNAME = "";

    /** The index in the RecordStore where language and username can be found. */
    public static final int LANGUAGE_INDEX = 1;
    public static final int USERNAME_INDEX = 2;

    private int index;
    private String value;

    /**
     * This method tries to read the content of the applications's recordstore. If this is the
     * first use of the appplication, this method creates the recordstore using default values.
     * Otherwise it reads the values and stores them in the variables LANGUAGE and USERNAME.
     */
    public void read() {
        try {
            RecordStore rs = RecordStore.openRecordStore(Configurations.RECORDSTORE_NAME, true);

            /* if this is the first time the application is run there is no recordstore yet: */
            if (rs.getNumRecords() == 0) {
                /* retrieving phone language to use as default: */
                String lang = (System.getProperty("microedition.locale")).toLowerCase();
                if (lang != null) LANGUAGE = lang;

                byte bytes[] = LANGUAGE.getBytes();
                rs.addRecord(bytes, 0, bytes.length);
                byte bytes2[] = "".getBytes();
                rs.addRecord(bytes2, 0, bytes2.length);
            }

            /* if the recordstore exist, the values in it is read: */
            else {
                String s = null;

                byte l[] = rs.getRecord(LANGUAGE_INDEX);
                LANGUAGE = new String(l);

                byte u[] = rs.getRecord(USERNAME_INDEX);
                if (u != null) s = new String(u);
                if (s != null && s.length() > 0) USERNAME = s;
            }
            rs.closeRecordStore();
        } catch (RecordStoreException rse) {
        }
    }

    /**
     * This method updates a record (recognised by the specified index) in the application's
     * recordstore and updates the value in this class.
     * 
     * @param index int The index in the recordstore that should be updated.
     * @param value String The new value to put in the recordstore.
     */
    public synchronized void write(int index, String value) {
        if (index == LANGUAGE_INDEX){
            LANGUAGE = value;
        } else {
            USERNAME = value;
        }
        this.index = index;
        this.value = value;
        (new Thread(this)).start();
    }

    /**
     * This method is used when writing a value to the RecordStore. Since this is a time
     * consuming operation compared to other things done at the same time, this runs in its own
     * Thread.
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        try {
            RecordStore rs = RecordStore.openRecordStore(RECORDSTORE_NAME, false);
            byte bytes[] = value.getBytes();
            rs.setRecord(index, bytes, 0, bytes.length);
            rs.closeRecordStore();
        } catch (RecordStoreException rse) {}
    }
}
