/*
 * Classname: Language
 * Version: 1.27
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package mymobilesite.controller;

/**
 * The Language class is abstract class to allow use of several languages. It holds variables
 * for the labels and texts in the application which should be initialized by any implementing
 * class which must be made to support a new language.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public abstract class Language {

    /** ID number of the error messages: */
    public static final int ERROR_IDPNAME_NOTFOUND = 0;
    public static final int ERROR_IDP_NOTFOUND = 1;
    public static final int ERROR_IDP_NOTTRUSTED = 2;
    public static final int ERROR_REPLAY_DETECTED = 3;
    public static final int ERROR_INVALID_MESSAGE = 4;
    public static final int ERROR_AUTH_NOTACCEPTED = 5;
    public static final int ERROR_IDP_LOSTCONTACT = 6;
    public static final int ERROR_IDP_FAULT = 7;
    public static final int ERROR_DS_LOSTCONTACT = 8;
    public static final int ERROR_DS_NORESULTS = 9;
    public static final int ERROR_WSP_LOSTCONTACT = 10;
    public static final int ERROR_WSP_NORESULTS = 11;
    public static final int ERROR_WSP_RECEIVEDFAULT = 12;
    public static final int ERROR_IDP_TIMESTAMPDEVIATION = 13;
    public static final int ERROR_DS_TIMESTAMPDEVIATION = 14;
    public static final int ERROR_WSP_TIMESTAMPDEVIATION = 15;

    /** Waiting messages: */
    public static final int WAITING_IDP_SESSION = 0;
    public static final int WAITING_VERIFYING_PWD = 1;
    public static final int WAITING_FINDING_WSP = 2;
    public static final int WAITING_GETTING_SERVICES = 3;

    /** List of supported languages: */
    public String languageList[];

    /** Labels used in the application: */
    public String label_applName;
    public String label_idp;
    public String label_un;
    public String label_pwd;
    public String label_help;
    public String label_language;
    public String label_providers;
    public String label_services;
    public String label_error;
    public String label_waiting;

    /** The text featured on the help screen: */
    public String help;

    /** The text on the command buttons: */
    public String command_ok;
    public String command_exit;
    public String command_settings;
    public String command_help;
    public String command_save;
    public String command_back;
    public String command_tryagain;

    /** List of error messages and waiting messages: */
    protected static String[] error_messages;
    protected static String[] waiting_messages;

    /**
     * The language constructor initializes the message arrays and populates the list of
     * languages.
     */
    public Language() {
        error_messages = new String[16];
        waiting_messages = new String[4];
        languageList = new String[] { "English (gb)", "Norsk (nb)" };
    }

    /**
     * This method creates a Language object and the exact class is determined by the language
     * in the Configurations class.
     * 
     * @return Language An instance of the Language class.
     */
    public static Language getLanguage() {
        Language lang;
        String l;
        if ((Configurations.LANGUAGE).indexOf("nb") != -1) {
            lang = new LanguageNB();
            l = "nb";
        } else if ((Configurations.LANGUAGE).indexOf("nn") != -1) {
            lang = new LanguageNB();
            l = "nn";
        } else if ((Configurations.LANGUAGE).indexOf("no") != -1) {
            lang = new LanguageNB();
            l = "nb";
        } else {
            lang = new LanguageGB();
            l = "gb";
        }
        (new Configurations()).write(Configurations.LANGUAGE_INDEX, l);
        return lang;
    }

    /**
     * This method is used to get the message in the correct language. It returns the error
     * message identified in the input.
     * 
     * @param errorCode int A number identifying the error message to return.
     * @return String The error message in the correct language.
     */
    public static String getErrorMsg(int errorCode) {
        return error_messages[errorCode];
    }

    /**
     * This method is used to get the message in the correct language. It returns the waiting
     * message identified in the input.
     * 
     * @param code int A number identifying the waiting message to return.
     * @return String The waiting message in the correct language.
     */
    public static String getWaitingMsg(int code) {
        return waiting_messages[code];
    }
}
