/*
 * Classname: LibertyProtocolListener
 * Version: 1.5
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package mymobilesite.controller;

import mymobilesite.communication.MessageException;

/**
 * The LibertyProtocolListener interface is a listener capable of receiving events concerning
 * the LibertyProtocol class.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public interface LibertyProtocolListener {
    
    /**
     * The implementation of this method should receive the result and handle it according to
     * the current state of the LibertyProtocol.
     * 
     * @param state int The state which the LibertyProtocol is currently in.
     * @param result Object The result from the LibertyProtocol to the listener.
     */
    public void resultsReady(int state, Object result);

    /**
     * The implementation of this method should receive the exception and take appropriate
     * action according to the error code and the state of the LibertyProvider.
     * 
     * @param state int The state which the LibertyProtocol is currently in.
     * @param me MessageException The MessageException that the LibertyProtocol encountered.
     */
    public void encounteredException(int state, MessageException me);

}
