/*
 * Classname: Log
 * Version: 1.2
 * Date: 20.05.2006
 * Copyright: Copyright (c) 2006 Gunn Olaussen, Kirsti N. Torgersen. All Rights Reserved.
 * 
 * This software is the intellectual property of Gunn Olaussen and Kirsti N. Torgersen 
 * (henceforth called the developers). It cannot be exploited financially used for any 
 * purpose by others than the developers, except in accordance with the terms of the 
 * agreement between Kantega AS and the developers.
 */

package loanfund;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

/**
 * The Log class is responsible for writing the log files when an exception occurs within the
 * system. The time of the occurance and the stack trace is printed to make it easier to
 * correct errors in the system.
 * 
 * @author Gunn Olaussen & Kirsti N. Torgersen
 */
public class Log {

    /**
     * The write message opens the log file if it exists or it creates a new file to which it
     * writes the stack trace of the exception.
     * 
     * @param exception Exception The exception which occured during execution
     */
    public static void write(Exception exception) {
        BufferedWriter writer = null;
        try {
            File logFile = new File("/home/apache-tomcat-5.5.17/logs/WSPlog.txt");
            if (!logFile.exists()) {
                logFile.createNewFile();
            }
            writer = new BufferedWriter(new FileWriter(logFile, true));
            writer.write("- " + new Date(System.currentTimeMillis()).toString()
                    + " ----------------------------------------------------------");
            writer.newLine();
            writer.write(exception.getClass() + ": " + exception.getMessage());
            writer.newLine();
            String classname;
            String methodname;
            int lineno;
            StackTraceElement[] trace = exception.getStackTrace();
            for (int i = 0; i < trace.length; i++) {
                classname = trace[i].getClassName();
                methodname = trace[i].getMethodName();
                lineno = trace[i].getLineNumber();
                writer.write("     Class: " + classname + " Method: " + methodname
                        + " Lineno: " + lineno);
                writer.newLine();
            }
            writer.write("--------------------------------------------------------"
                    + "----------------------------------");
            writer.newLine();
            writer.flush();
        } catch (Exception e) {
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                }
            }
        }
    }
}
